from enk.filegen import build_files
import sys
import stat
import textwrap

# Note - where definitions use non-ASCII chacaters, they need to be Unicode,
# hence the seemingly-random scattering of u"""...""".

def test_empty_file(tmpdir):
    defn = textwrap.dedent("""
        name: foo
    """)
    build_files(defn, cwd=str(tmpdir))
    assert (tmpdir / 'foo').check()
    assert (tmpdir / 'foo').size() == 0

def test_empty_directory(tmpdir):
    defn = textwrap.dedent("""
        name: foo
        directory: yes
    """)
    build_files(defn, cwd=str(tmpdir))
    assert (tmpdir / 'foo').check(dir=1)

def test_trailing_slash(tmpdir):
    defn = textwrap.dedent("""
        name: foo/
    """)
    build_files(defn, cwd=str(tmpdir))
    assert (tmpdir / 'foo').check(dir=1)

def test_set_content(tmpdir):
    # Also tests that a Python 2 string can be used for content when it
    # is pure ASCII.
    defn = textwrap.dedent("""
        name: foo
        content: bar
    """)
    build_files(defn, cwd=str(tmpdir))
    assert (tmpdir / 'foo').read() == 'bar'

def test_encoding(tmpdir):
    defn = textwrap.dedent(u"""
        name: foo
        content: \xa3
        encoding: latin-1
    """)
    build_files(defn, cwd=str(tmpdir))
    assert (tmpdir / 'foo').read_text("latin-1") == '\xa3'

def test_encoding_utf8(tmpdir):
    defn = textwrap.dedent(u"""
        name: foo
        content: \xa3
        encoding: utf-8
    """)
    build_files(defn, cwd=str(tmpdir))
    assert (tmpdir / 'foo').read_text("utf-8") == '\xa3'

def test_default_encoding_is_utf8(tmpdir):
    defn = textwrap.dedent(u"""
        name: foo
        content: \xa3
    """)
    build_files(defn, cwd=str(tmpdir))
    assert (tmpdir / 'foo').read_text("utf-8") == '\xa3'

def test_non_ascii_filename(tmpdir):
    defn = textwrap.dedent(u"""
        name: foo\xa3
    """)
    build_files(defn, cwd=str(tmpdir))
    assert (tmpdir / 'foo\xa3').check()

def test_multiple_files(tmpdir):
    defn = textwrap.dedent("""
        name: foo
        ---
        name: bar
    """)
    build_files(defn, cwd=str(tmpdir))
    assert (tmpdir / 'foo').check()
    assert (tmpdir / 'bar').check()

def test_indented_content(tmpdir):
    defn = textwrap.dedent("""
        name: foo
        content: |
          Line 1
            Line 2
          Line 3
    """)
    build_files(defn, cwd=str(tmpdir))
    assert (tmpdir / 'foo').read() == "Line 1\n  Line 2\nLine 3\n"

def test_executable(tmpdir):
    defn = textwrap.dedent("""
        name: foo
        executable: yes
    """)
    build_files(defn, cwd=str(tmpdir))
    if not sys.platform.startswith('win'):
        # Executable bit is not relevant on Windows
        assert ((tmpdir / 'foo').stat().mode & stat.S_IXUSR) != 0



import sys
import shutil

PY2_DEFAULT_TERMINAL_SIZE = 80
last_written = 0

def overwrite_line(text, newline=False):
    global last_written
    if newline:
        sys.stdout.write('\n')
        last_written = 0
    if last_written > 0:
        sys.stdout.write('\r')
    text_len = len(text)
    sys.stdout.write(text)
    if text_len < last_written:
        clear = last_written - text_len
        sys.stdout.write(' ' * clear)
        sys.stdout.write('\b' * clear)

    last_written = text_len
    sys.stdout.flush()

def append_line(text):
    global last_written
    last_written += len(text)
    sys.stdout.write(text)
    sys.stdout.flush()

class ScreenLine:
    def __init__(self, final=""):
        try:
            self.length, ignore = shutil.get_terminal_size()
        except AttributeError:
            # Python 2 has no shutil.get_terminal_size()
            self.length = PY2_DEFAULT_TERMINAL_SIZE
        # Assume the line is dirty, so clear it on first write
        self.written = self.length - 1
        self.final = final
    def show(self, line):
        # Tidy up the input. Remove trailing space, and ignore all except the
        # last line
        line = line.rstrip()
        line = line.split("\n")[-1]
        # Truncate to screen width or less
        # Leave 4 chars, 3 for ... and 1 to avoid wrapping
        if len(line) > self.length - 4:
            line = line[:self.length - 4] + '...'
        sys.stdout.write('\r')
        text_len = len(line)
        sys.stdout.write(line)
        if text_len < self.written:
            clear = self.written - text_len
            sys.stdout.write(' ' * clear)
            sys.stdout.write('\b' * clear)
        self.written = text_len
        sys.stdout.flush()
    def finish(self, text=None):
        if not text:
            text = self.final
        if text:
            sys.stdout.write(" ")
        sys.stdout.write(text + "\n")
        sys.stdout.flush()
        self.written = 0
    def __enter__(self):
        return self
    def __exit__(self, exc_type, exc_value, tb):
        self.finish()



from enk.console import overwrite_line, append_line, ScreenLine
import shutil

def patch_termsize(monkeypatch):
    def dummy_termsize():
        return 10, 10

    if hasattr(shutil, 'get_terminal_size'):
        monkeypatch.setattr('shutil.get_terminal_size', dummy_termsize)
    else:
        monkeypatch.setattr('enk.console.PY2_DEFAULT_TERMINAL_SIZE', 10)

def test_overwrite(capsys):
    # New line - no CR
    overwrite_line("Test 1")
    out, err = capsys.readouterr()
    assert out == 'Test 1'
    overwrite_line("Test 2")
    out, err = capsys.readouterr()
    assert out == '\rTest 2'
    overwrite_line("Test 12")
    out, err = capsys.readouterr()
    assert out == '\rTest 12'
    overwrite_line("Test 1")
    out, err = capsys.readouterr()
    assert out == '\rTest 1 \b'
    append_line("OK")
    out, err = capsys.readouterr()
    assert out == 'OK'
    # New line - no CR
    overwrite_line("Test 1", newline=True)
    out, err = capsys.readouterr()
    assert out == '\nTest 1'

def padding(n):
    return (" " * n) + ("\b" * n)

def test_screenline_init(monkeypatch, capsys):
    patch_termsize(monkeypatch)
    s = ScreenLine()
    s.show("")
    out, err = capsys.readouterr()
    assert out == "\r" + padding(9)
    assert err == ""

def test_screenline_write(monkeypatch, capsys):
    patch_termsize(monkeypatch)
    s = ScreenLine()
    s.show("OK")
    out, err = capsys.readouterr()
    assert out == "\r" + "OK" + padding(7)
    assert err == ""

def test_screenline_optimise_second_write(monkeypatch, capsys):
    patch_termsize(monkeypatch)
    s = ScreenLine()
    s.show("")
    out, err = capsys.readouterr()
    s.show("Hello")
    out, err = capsys.readouterr()
    assert out == "\r" + "Hello"
    assert err == ""

def test_screenline_toolong(monkeypatch, capsys):
    patch_termsize(monkeypatch)
    s = ScreenLine()
    s.show("")
    out, err = capsys.readouterr()
    s.show("This is longer than six characters")
    out, err = capsys.readouterr()
    assert out == "\r" + "This i..."
    assert err == ""

def test_screenline_contextmgr(monkeypatch, capsys):
    patch_termsize(monkeypatch)
    with ScreenLine() as s:
        s.show("Test")
    out, err = capsys.readouterr()
    assert out == "\r" + "Test" + padding(5) + "\n"
    assert err == ""

def test_screenline_final(monkeypatch, capsys):
    patch_termsize(monkeypatch)
    with ScreenLine(final="OK") as s:
        s.show("Test")
    out, err = capsys.readouterr()
    assert out == "\r" + "Test" + padding(5) + " OK\n"
    assert err == ""



from subprocess import Popen, PIPE, STDOUT

def process_lines(args, **kw):
    # Ignore args we will supply
    kw['args'] = args
    kw['stdout'] = PIPE
    kw['stderr'] = STDOUT
    kw['universal_newlines'] = True

    proc = Popen(**kw)
    try:
        for line in proc.stdout:
            yield line
    finally:
        if proc.stdout:
            proc.stdout.close()
        if proc.stderr:
            proc.stderr.close()
        if proc.stdin:
            proc.stdin.close()
        # Wait for the process to terminate, to avoid zombies.
        proc.wait()



import os
import io
import stat
import yaml

try:
    unicode
except NameError:
    def unicode(s): return s

def ensuredir(name):
    """Make sure directory name exists.

    Code lifted (and hugely simplified) from os.makedirs.
    
    """
    head, tail = os.path.split(name)
    if not tail:
        head, tail = os.path.split(head)
    if head and tail and not os.path.exists(head):
        ensuredir(head)
    if not os.path.isdir(name):
        os.mkdir(name)

def build_one_file(defn, cwd=None):
    filename = defn['name']
    if cwd:
        filename = os.path.join(cwd, filename)
    dirname, basename = os.path.split(filename)
    ensuredir(dirname)
    if not basename:
        return
    if defn.get('directory'):
        os.mkdir(filename)
        return
    with io.open(filename, 'w', encoding=defn.get('encoding', 'utf-8')) as f:
        f.write(unicode(defn.get('content', u'')))
    if defn.get('executable'):
        os.chmod(filename, os.stat(filename).st_mode | stat.S_IXUSR)

def build_files(defn, cwd=None):
    for defn in yaml.safe_load_all(defn):
        build_one_file(defn, cwd)
